<?php

namespace MobiCustomization\AutoCouponGeneration\Model;

use Magento\SalesRule\Api\Data\RuleInterface;
use MobiCustomization\AutoCouponGeneration\Helper\Email as CouponEmail;
use Magento\SalesRule\Model\Rule\Condition\Combine;
use Magento\SalesRule\Model\Rule\Condition\Product\Found;
use Magento\SalesRule\Model\Rule\Condition\Product;

class AutoGenerateCoupon {
    protected $ruleFactory;
    protected $productRuleFactory;
    protected $foundProductRuleFactory;
    protected $ruleResource;
    protected $generationSpecFactory;
    protected $couponManagementService;
    protected $websiteCollectionFactory;
    protected $_date;
    protected $rule;
    protected $couponEmail;
    protected $emailCouponData;

    public function __construct(
        \Magento\SalesRule\Model\RuleFactory $ruleFactory,
        \Magento\SalesRule\Model\Rule\Condition\ProductFactory $productRuleFactory,
        \Magento\SalesRule\Model\Rule\Condition\Product\FoundFactory $foundProductRuleFactory,
        \Magento\SalesRule\Model\ResourceModel\Rule $ruleResource,
        \Magento\SalesRule\Api\Data\CouponGenerationSpecInterfaceFactory $generationSpecFactory,
        \Magento\SalesRule\Model\Service\CouponManagementService $couponManagementService,
        \Magento\Store\Model\ResourceModel\Website\CollectionFactory $websiteCollectionFactory,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        RuleInterface $rule,
        CouponEmail $couponEmail
    ) {
        $this->ruleFactory = $ruleFactory;
        $this->productRuleFactory = $productRuleFactory;
        $this->foundProductRuleFactory = $foundProductRuleFactory;
        $this->ruleResource = $ruleResource;
        $this->generationSpecFactory = $generationSpecFactory;
        $this->couponManagementService = $couponManagementService;
        $this->websiteCollectionFactory = $websiteCollectionFactory;
        $this->_date =  $date;
        $this->rule = $rule;
        $this->couponEmail = $couponEmail;
        $this->emailCouponData = "";
    }

    public function createCartRules($customer_id, $order_id, $discountAmount, $customer_email, $customer_name)
    {
        if(is_null($customer_id) || $customer_id === ""){
            return;
        }        
        
        $websiteIds = array();
        $noOfCounpons = 4;
        $discountAmount = $discountAmount * 2; // multiply to product price like 2x

        $websiteCollection = $this->websiteCollectionFactory->create();
        foreach ($websiteCollection as $website) {
            array_push($websiteIds,$website->getId());
        }
        $perCouponDiscountAmount = round(($discountAmount/$noOfCounpons), 2);

        for ($i=0; $i < $noOfCounpons; $i++) { 
            $this->createMemberShipRules($customer_id, $order_id, $perCouponDiscountAmount, $websiteIds, $i);
        }

        if($this->emailCouponData !== ""){
            $this->couponEmail->SendEmail(
                    $customer_name, 
                    $customer_email,
                    $this->emailCouponData
                );
        }

    }

    public function createMemberShipRules($customer_id, $order_id, $perCouponDiscountAmount, $websiteIds, $couponIndex)
    {
        //$expiryDuration = "+30 days";
        $couponCode = strtoupper(substr(sha1(microtime()), rand(0, 4), 12));
        $couponCode = implode("-", str_split($couponCode, 4)); 

        $expiryToDays = ($couponIndex + 1) * 7; 
        $expiryFromDays = $expiryToDays - 7; 
        $expiryToDays = $expiryToDays - 1; // make one day priore as expiry
        $expiryToDuration = "+" . $expiryToDays . " days";
        $expiryFromDuration = "+" . $expiryFromDays . " days";
        $expiryToDate = $this->_date->gmtDate('Y-m-d',$this->_date->timestamp($expiryToDuration));
        $expiryFromDate = $this->_date->gmtDate('Y-m-d',$this->_date->timestamp($expiryFromDuration));

        
        $mobi_writer = new \Zend_Log_Writer_Stream(BP . '/var/log/AutoGenerateCoupon.log');
        $mobi_logger = new \Zend_Log();
        $mobi_logger->addWriter($mobi_writer);
        $mobi_logger->info("input ". $customer_id ." - ". $order_id ." - ". $perCouponDiscountAmount . " - ". $expiryFromDate . " - " . $expiryToDate);


        $shoppingCartPriceRule = $this->ruleFactory->create();

        $shoppingCartPriceRule->setName('Membership Discount C#'. $customer_id) // Rules name
            ->setDescription("Membership Discount")
            ->setFromDate($expiryFromDate)
            ->setToDate($expiryToDate)
            ->setUsesPerCustomer(1)
            ->setCustomerGroupIds([1]) // for logged user only
            ->setIsActive(true)
            ->setStopRulesProcessing(true)
            ->setIsAdvanced(true)
            ->setSimpleAction(RuleInterface::DISCOUNT_ACTION_FIXED_AMOUNT_FOR_CART)
            ->setDiscountAmount($perCouponDiscountAmount)
            ->setDiscountQty(1)
            //->setDiscountStep('0')
            //->setSimpleFreeShipping('0')
            //->setApplyToShipping('0')
            //->setTimesUsed('0')
            //->setIsRss('0')
            ->setWebsiteIds($websiteIds)
            //->setCouponType(RuleInterface::COUPON_TYPE_SPECIFIC_COUPON)
            ->setCouponType('2')
            ->setUseAutoGeneration(false)
            ->setCouponCode($couponCode)
            ->setUsesPerCoupon(1)
            ->setUsesPerCustomer(1)
            ->setData("mobi_customer_id", $customer_id)
            ->setData("mobi_remark", 'Membership Coupon #'. $couponIndex + 1 .' for Customer Id #'. $customer_id . " against order Id #". $order_id)
            ->setData("is_membership_discount_coupon", 1);

            

            $shoppingCartPriceRule->getConditions()->loadArray(
                [
                    'type' => Combine::class,
                    'attribute' => null,
                    'operator' => null,
                    'value' => '0',
                    'is_value_processed' => null,
                    'aggregator' => 'any',
                    'conditions' => [
                            [
                                'type' => Found::class,
                                'attribute' => null,
                                'operator' => null,
                                'value' => "1",
                                'is_value_processed' => null,
                                'aggregator' => 'all',
                                'conditions' => [
                                    [
                                        'type' => Product::class,
                                        'attribute' => 'is_membership_product',
                                        'operator' => '==',
                                        'value' => "1",
                                        'is_value_processed' => false,
                                        'attribute_scope' => '',
                                        
                                    ]
                                ]
                            ]
                    ],
                ]
            );

        try {
            $this->ruleResource->save($shoppingCartPriceRule);
            $this->emailCouponData = $this->emailCouponData . "<tr><td>" . $couponCode . " </td><td>$". $perCouponDiscountAmount . "</td><td>". $expiryFromDate . " - " . $expiryToDate . " </td></tr>";
            $mobi_logger->info("Created Coupons ". $couponCode ." - ". $perCouponDiscountAmount ." - ". $expiryFromDate . " - " . $expiryToDate);

        }catch (\Magento\Framework\Exception\LocalizedException $ex) {
            $mobi_logger->info($ex->getMessage());
        }        
    }

}