<?php
/**
 * MobiCustomization_AutoCouponGeneration
 *
 * @category  PHP
 * @package   MobiCustomization_AutoCouponGeneration
 * @author    MobiCustomization <info@mobicommerce.net>
 * @copyright 2022 Copyright (c) MobiCommerce
 * @link      https://www.mobicommerce.net/
 */


namespace MobiCustomization\AutoCouponGeneration\Observer\Frontend\Sales;

use Magento\Framework\Event\ObserverInterface;
use MobiCustomization\AutoCouponGeneration\Model\AutoGenerateCoupon as AutoGenerateCoupon;

class CreateMembershipDiscountCoupons implements ObserverInterface
{
    protected $autoGenerateCoupon;  

    public function __construct(
        AutoGenerateCoupon $autoGenerateCoupon,        
    ){
        $this->autoGenerateCoupon = $autoGenerateCoupon;
    }
 
    public function execute(\Magento\Framework\Event\Observer $observer )
    {
        $mobi_writer = new \Zend_Log_Writer_Stream(BP . '/var/log/AutoGenerateCoupon.log');
        $mobi_logger = new \Zend_Log();
        $mobi_logger->addWriter($mobi_writer);

        try {
            $order = $observer->getEvent()->getOrder();
            $order_id = $order->getIncrementId();
            $customer_email = $order->getCustomerEmail();
            $customer_id = $order->getCustomerId();
            if($order->getCustomerName()){
                $customer_name = $order->getCustomerName();
            } else{
                $customer_name = $order->getBillingAddress()->getName();
            }
            

            $isMemberShipProduct = false;
            // check logged in user order, if than process
            if(!is_null($customer_id)){
                //$mobi_logger->info("Order Id - " . $order_id . " Customer Id - " . $customer_id);
                //check all products in order, if any product as membership than create coupons
                foreach($order->getAllItems() as $item){
                    if($item->getIsMembershipProduct() === 1 || $item->getIsMembershipProduct() === "1"){
                        $isMemberShipProduct = true;                        
                        $discountAmount = $item->getRowTotalInclTax();

                    }
                    //$mobi_logger->info($order_id . " - is Membership - " . $isMemberShipProduct);
                }
                if($isMemberShipProduct === true){
                    //$mobi_logger->info($order_id . " create coupon as order has membership product " . $isMemberShipProduct);
                    //create coupon as order has membership product
                    $this->autoGenerateCoupon->createCartRules($customer_id, $order_id, $discountAmount, $customer_email, $customer_name);
                }
            }

        } catch (\Exception $e) {
            $mobi_logger->info($e->getMessage());
        }
    }
}