<?php
/**
 * MobiCustomization_AutoCouponGeneration
 *
 * @category  PHP
 * @package   MobiCustomization_AutoCouponGeneration
 * @author    MobiCustomization <info@mobicommerce.net>
 * @copyright 2022 Copyright (c) MobiCommerce
 * @link      https://www.mobicommerce.net/
 */

//https://community.magento.com/t5/Magento-2-x-Programming/Magento-2-add-custom-attributes-in-quote-and-order/td-p/477189 


namespace MobiCustomization\MarkMembershipCouponInactive\Observer\Frontend\Sales;

use Magento\Framework\Event\ObserverInterface;
use Exception;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\SalesRule\Api\Data\CouponInterface;
use Magento\SalesRule\Api\CouponRepositoryInterface;
use Magento\SalesRule\Api\Data\RuleInterface;
use Magento\SalesRule\Api\RuleRepositoryInterface;

class MarkMembershipCouponInactive implements ObserverInterface
{
    private $couponRepository;
    private $searchCriteriaBuilder;
    protected $ruleRepository;
    protected $rule;

    public function __construct(
        CouponRepositoryInterface $couponRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        RuleRepositoryInterface $ruleRepository,
        \Magento\SalesRule\Model\Rule $rule             
    ){
        $this->couponRepository = $couponRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;  
        $this->ruleRepository = $ruleRepository;
        $this->rule = $rule;

    }
 
    public function execute(\Magento\Framework\Event\Observer $observer )
    {
        $mobi_writer = new \Zend_Log_Writer_Stream(BP . '/var/log/MarkMembershipCouponInactive.log');
        $mobi_logger = new \Zend_Log();
        $mobi_logger->addWriter($mobi_writer);

        try {
            $order = $observer->getEvent()->getOrder();
            $order_id = $order->getIncrementId();
            $appliedCouponCode = $order->getCouponCode();
            //$customer_email = $order->getCustomerEmail();
            $customer_id = $order->getCustomerId();

            $mobi_logger->info("Order Id - ".  $order_id .  " Customer Id - " . $customer_id . " Applied Coupon ". $appliedCouponCode );

            if(!is_null($customer_id) && !is_null($appliedCouponCode)){
                $couponData = $this->getCouponDataByCode($appliedCouponCode);
                foreach ($couponData as $coupon) {
                    $ruleId = $coupon->getRuleId();
                    $ruleData = $this->getRuledata($ruleId);
                    if(!is_null($ruleData->getMobiCustomerId()) && !is_null($ruleData->getIsMembershipDiscountCoupon())){
                        if($customer_id === $ruleData->getMobiCustomerId()){
                            $mobi_logger->info("Update Rule status here");
                            $ruleData->setIsActive(false);
                            $ruleData->setMobiRemark("Coupon is used online, Order Id #".  $order_id . "<br>". $this->addNewLine(). $this->addNewLine() . $ruleData->getMobiRemark());
                            $ruleData->save();
                        }
                    }
                    $mobi_logger->info("Rule Id - ". $ruleId);
                    $mobi_logger->info("Rule Name - ". $ruleData->getName());
                    $mobi_logger->info("Rule Customer Id - ". $ruleData->getMobiCustomerId());
                    $mobi_logger->info("Rule is Membership Discount - ". $ruleData->getIsMembershipDiscountCoupon());

                }                
            }

        } catch (\Exception $e) {
            $mobi_logger->info($e->getMessage());
        }
    }


    public function getCouponDataByCode($couponCode)
    {
        $couponData = null;
        $searchCriteria = $this->searchCriteriaBuilder->addFilter('code', $couponCode)->create();
        try {
            $couponList = $this->couponRepository->getList($searchCriteria);
            if ($couponList->getTotalCount()) {
                $couponData = $couponList->getItems();
            }

        } catch (Exception $exception) {
            $this->logger->error($exception->getMessage());
        }
        return $couponData;
    }

    public function getRuledata($ruleId)
    {
        try {
            $salesRule = $this->rule->load($ruleId);
        } catch (Exception $exception) {
            $this->logger->error($exception->getMessage());
        }
        return $salesRule;
    }

    public function addNewLine(){
        return "
";
    }

}