<?php

/**
 * MobiCustomization_MembershipProduct
 *
 * @category  PHP
 * @package   MobiCustomization_MembershipProduct
 * @author    MobiCustomization <info@mobicommerce.net>
 * @copyright 2022 Copyright (c) MobiCommerce
 * @link      https://www.mobicommerce.net/
 */

namespace MobiCustomization\MembershipProduct\Setup\Patch\Data;

use Magento\Catalog\Model\Config;
use Magento\Eav\Api\AttributeManagementInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;
use Magento\Framework\Setup\Patch\PatchRevertableInterface;
use Magento\Catalog\Model\Product;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Eav\Setup\EavSetup;
use Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface;
use Magento\Eav\Model\Entity\Attribute\Source\Boolean;


class CreateProductAttributes implements
    DataPatchInterface,
    PatchRevertableInterface
{
    /**
     * @var ModuleDataSetupInterface
     */
    private ModuleDataSetupInterface $moduleDataSetup;

    /**
     * @var EavSetupFactory
     */
    private EavSetupFactory $eavSetupFactory;

    /**
     * Constructor
     *
     * @param ModuleDataSetupInterface $moduleDataSetup
     * @param EavSetupFactory $eavSetupFactory
     */

    /**
     * @var Config
     */
    private $config;
    /**
     * @var AttributeManagementInterface
     */
    private $attributeManagement;

    public function __construct(
        ModuleDataSetupInterface $moduleDataSetup,
        EavSetupFactory $eavSetupFactory,
        Config $config,
        AttributeManagementInterface $attributeManagement
    ) {
        $this->moduleDataSetup = $moduleDataSetup;
        $this->eavSetupFactory = $eavSetupFactory;
        $this->config = $config;
        $this->attributeManagement = $attributeManagement;      
    }

    /**
     * @inheritDoc
     * @throws LocalizedException
     */
    public function apply()
    {
        $this->moduleDataSetup->getConnection()->startSetup();

        $this->createRequiredAttributes();

        $this->moduleDataSetup->getConnection()->endSetup();
    }

    /**
     * @inheritDoc
     */
    public static function getDependencies(): array
    {
        return [];
    }

    /**
     * @inheritDoc
     */
    public function revert()
    {
        $this->moduleDataSetup->getConnection()->startSetup();
        $this->moduleDataSetup->getConnection()->endSetup();
    }

    /**
     * @inheritDoc
     */
    public function getAliases(): array
    {
        return [];
    }

    /**
     * Add required attributes
     *
     * @return void
     * @throws LocalizedException
     */
    private function createRequiredAttributes() : void
    {
        $eavSetup = $this->eavSetupFactory->create(['setup' => $this->moduleDataSetup]);

        /**
         * This is to define whether the product is Membership product or not, default will be false
         */
        $eavSetup->addAttribute(Product::ENTITY, 'is_membership_product', [
            'type' => 'int',
            'backend' => '',
            'frontend' => '',
            'label' => 'Is Membership Product',
            'input' => 'boolean',
            'class' => '',
            'source' => Boolean::class,
            'global' => ScopedAttributeInterface::SCOPE_GLOBAL,
            'visible' => true,
            'required' => false,
            'user_defined' => false,
            'default' => '0',
            'searchable' => false,
            'filterable' => false,
            'comparable' => false,
            'sort_order' => 250,
            'visible_on_front' => false,
            'used_in_product_listing' => true,
            'used_for_promo_rules' => true,   
            'is_used_in_grid' => true,
            'is_filterable_in_grid' => true,
            'unique' => false,
        ]);

        $attributeSetIds = $eavSetup->getAllAttributeSetIds(\Magento\Catalog\Model\Product::ENTITY);

        foreach ($attributeSetIds as $attributeSetId) {
            $groupId = $this->config->getAttributeGroupId($attributeSetId, 'Product Details');
            $this->attributeManagement->assign(
                \Magento\Catalog\Model\Product::ENTITY,
                $attributeSetId,
                $groupId,
                'is_membership_product',
                250
            );
        }
       
    }
}
